﻿require.config({
    baseUrl: "/",
    paths: {
        "moment": "/sitecore/shell/client/Speak/Assets/lib/ui/1.1/deps/moment/moment.min"
    }
});

define(["sitecore", "moment", "userProfile"], function (_sc, moment) {
    var models = _sc.D3.models;

    models.colors.standardColors = ["#DF7C77", "#EFBDBB"].concat(models.colors.standardColors);

    models.formatterConfig.dateFormatter = function (options) {
        return function (date) {
            return getFormattedDate(date);
        };
    };

    models.utils.ellipse = (function () { //Adjust for angled labels
        var cached_function = models.utils.ellipse;
        return function () {
            return cached_function.apply(this, (((((arguments[0] || [])[0] || [])[0] || []).classList) || document.body.classList).contains("nv-legend-text") ? arguments : [arguments[0], 70, 2, 2]);
        };
    })();

    models.nvd3Axis.removeOverlappingLabels = function (canvas, hasBarInCombinationChart, dataLength, chartType) {
        return dataLength;
    };

    ["Area", "Column"].forEach(function (chartType) {
        _sc.D3.components[chartType].prototype.initChart = (function () {
            var cached_function = _sc.D3.components[chartType].prototype.initChart;
            return function () {
                this.ChartProperties.margin.bottom += 15;
                return cached_function.apply(this, arguments);
            };
        })();
    });

    getTypes = function (includeComponent) {
        var types = ["Traffic", "AverageTev", "AverageTevByPage", "BounceRate"];
        return includeComponent ? types.concat("Component") : types;
    }

    getFormattedDate = function (date) {
        return date.toLocaleDateString(__speak_config_active_culture,
            {
                year: 'numeric',
                month: 'short',
                day: 'numeric'
            })
    }

    var AutoPersonalizationStandard = _sc.Definitions.App.extend({

        initialize: function () {
        },

        initialized: function () {
            this.initializeErrorListeners();
            this.reparentProgressIndicators();
            this.populateLists();
            this.initializeListenersForLabelUpdate();
            this.setListBoxSelectedItems();
            this.initializeListenersForChartLoad();
            this.refreshCharts();
            this.initializeComponentListBoxListeners();
            this.regulateDropDowns();
            this.getWarnings();

            this.getData("ComponentDropdown", this.AverageTevByPageListBox.get("selectedValue"), null, "ComponentListBox", "ComponentDataProvider");
        },

        initializeComponentListBoxListeners: function () {
            this.ComponentListBox.on("change:items", function () {
                var selVal = null;
                this.trimListBoxSize(["Component"]);

                if (this.ComponentListBox.attributes.items.length) {
                    var startIndex = this.ComponentListBox.attributes.items[0].id == "sc-optGroupUnselectable" ? 1 : 0;
                    this.quietUpdate = true;

                    selVal = this.ComponentListBox.attributes.items[startIndex].id;
                }
                else {
                    this.ComponentDropDown.set("text", "");
                }
                this.ComponentDropDown.set("isEnabled", this.ComponentListBox.attributes.items.length);

                var initialId = this.State.attributes.actionsStatus;
                if (typeof (initialId) != "string") {
                    initialId = "";
                }
                initialId = initialId.replaceAll("\\", "");
                if (initialId[0] == '"') {
                    initialId = initialId.slice(1, -1);
                }

                if (this.ComponentListBox.attributes.items.filter(function (item) {
                    return item.id == initialId;
                }).length) {
                    selVal = initialId;
                }
                this.ComponentListBox.set("selectedValue", selVal);

                this.quietUpdate = false;
                this.justChangedDates = true;
            }.bind(this));
        },

        initializeErrorListeners: function () {
            this.MessageBarDataProvider.on("error", this.showError.bind(this));
            this.TrafficChartDataProvider.on("error", this.showError.bind(this));
            this.ComponentDataProvider.on("error", this.showError.bind(this));
            this.AverageTevChartDataProvider.on("error", this.showError.bind(this));
            this.AverageTevByPageChartDataProvider.on("error", this.showError.bind(this));
            this.BounceRateChartDataProvider.on("error", this.showError.bind(this));
        },

        getWarnings: function () {
            this.getData("Warning", null, null, "AutoPersStandardMessageBar", "MessageBarDataProvider");
        },

        showError: function () {
            this.AutoPersStandardMessageBar.addMessage("error", { text: "A request to the server has failed", actions: [], closable: true })
        },

        refreshCharts() {
            this.toggleAverageTevByPageProgressIndicator(true);
            getTypes().forEach(function (chart) {
                chart != "AverageTevByPage" && this.loadChart(chart);
            }.bind(this));
        },

        initializeListenersForLabelUpdate: function () {
            getTypes(true).forEach(function (type) {
                this[type + "ListBox"].on("change:selectedItem", function () {
                    var item = this[type + "ListBox"].attributes.selectedItem || {};
                    var text = item.range || item.label;
                    this[type + "DropDown"].viewModel.close();
                    text && this[type + "DropDown"].set("text", text);
                }.bind(this));
            }.bind(this));
        },

        initializeListenersForChartLoad: function () {
            getTypes(true).forEach(function (type) {
                this[type + "ListBox"].on("change:selectedItem", this.loadChart.bind(this, type));
            }.bind(this));
            this.ComponentListBox.on("change:selectedItem", this.setSelectedComponentInUserProfile.bind(this));
            this.AverageTevByPageListBox.on("change:selectedItem", this.toggleAverageTevByPageProgressIndicator.bind(this, true));
        },

        toggleAverageTevByPageProgressIndicator: function (on) {
            this.AverageTevByPageProgressIndicator.set("isBusy", on);
        },

        setSelectedComponentInUserProfile: function () {
            this.State.attributes.actionsStatus = this.ComponentListBox.get("selectedValue");
            this.State.viewModel.setFavoritesInUserProfile();
        },

        regulateDropDowns: function () {
            var $dropDownButtons = $('.sc div[data-sc-id="AutoPersStandardDetailsTabBorder"] .sc-dropdownbutton button');
            $dropDownButtons.on("click", function (e) {
                var $target = $(e.target).closest('.sc-dropdownbutton');
                getTypes(true).forEach(function (type) {
                    var vm = this[type + "DropDown"].viewModel;
                    !vm.$el.is($target) && vm.close();
                }.bind(this));
            }.bind(this));
        },

        populateLists: function () {
            var segments = this.getTimeSegments(23, false);
            this.TrafficListBox.set("items", segments);
            this.AverageTevListBox.set("items", segments);

            var weeks = this.getTimeSegments(7, true);
            this.AverageTevByPageListBox.set("items", weeks);
            this.BounceRateListBox.set("items", weeks);

            this.trimListBoxSize();
        },

        trimListBoxSize: function (listBox) {
            function shorten() {
                return this.options.length;
            }
            var $border = $('.sc div[data-sc-id="AutoPersStandardDetailsTabBorder"]');
            var types = listBox ? [listBox] : getTypes(true);
            types.forEach(function (type) {
                this[type + "DropDown"].set("isEnabled", true);
                $border.find('select[data-sc-id="' + type + 'ListBox"]').attr("size", shorten);
            }.bind(this));
        },

        setListBoxSelectedItems: function () {
            getTypes().forEach(function (type) {
                var listBoxId = type + "ListBox";
                this[listBoxId].set("selectedValue", this[listBoxId].attributes.items[0].value);
            }.bind(this));
            return;
        },

        getTimeSegments: function (days, syncToNaturalWeek) {
            var segments = [];
            var t = moment().subtract(90, 'days').startOf('day');
            if (syncToNaturalWeek) {
                t = t.startOf('week').add(1, 'week');
            }

            while (t.isBefore()) {
                segments.push({ start: t.clone() });
                t.add(days, 'days');
            }

            for (var i = 0; i < segments.length; i++) {
                var seg = segments[i];
                seg.end = i == segments.length - 1 ? moment().clone() : segments[i + 1].start.clone().subtract(1, "days");
                var format = "YYYY-MM-DD";
                seg.value = JSON.stringify({
                    start: seg.start.format(format),
                    end: seg.end.format(format)
                });
                seg.range = getFormattedDate(seg.start.toDate()) + " - " + getFormattedDate(seg.end.toDate());

            }
            return segments.reverse();
        },

        reparentProgressIndicators: function () {
            getTypes().forEach(function (type) {
                $('[data-sc-id="' + type + 'Border"]').append($('[data-sc-id="' + type + 'ProgressIndicator"]'));
            }.bind(this));
        },

        loadChart: function (chart) {
            if (chart == "Component" && !this.quietUpdate && this.justChangedDates) {
                this.justChangedDates = false;
                return;
            }
            if (chart == "AverageTevByPage") {
                this.AverageTevByPageDropDown.viewModel.close();
                this.ComponentDropDown.viewModel.close();
                this.justChangedDates = false;
                this.getData("ComponentDropdown", this.AverageTevByPageListBox.get("selectedValue"), null, "ComponentListBox", "ComponentDataProvider");
                return;
            }

            if (chart == "Component") {
                var selectedVal = this.ComponentListBox.attributes.items.length ? this.ComponentListBox.get("selectedValue") : null;
                if (selectedVal) {
                    this.getData("AverageTevByPage", this.AverageTevByPageListBox.get("selectedValue"), selectedVal);
                } else {
                }

                this.AverageTevByPageDropDown.viewModel.close();
                this.ComponentDropDown.viewModel.close();
                return;
            }

            this.getData(chart, this[chart + "ListBox"].get("selectedValue"));
            this[chart + "DropDown"].viewModel.close();
            this.ComponentDropDown.viewModel.close();
        },

        getData: function (query, range, ids, targetComponentId, dataProviderId) {
            if (!targetComponentId) {
                targetComponentId = query + "Chart";
                dataProviderId = query + "ChartDataProvider";
            }

            var parameters = {};
            if (query) {
                parameters.type = query;
            };
            if (range) {
                range = JSON.parse(range);
                parameters.start = range.start;
                parameters.end = range.end;
            };

            if (ids) {
                ids = JSON.parse(ids);
                parameters.componentid = ids.componentid;
                parameters.datasourceid = ids.datasourceid;
            };

            var requestOptions = {
                url: this[dataProviderId].get("dataUrl") + '?' + $.param(parameters),
                onSuccess: function (data) {
                    this.setData(data, targetComponentId);
                }.bind(this),
                onFailure: function () {
                    console.log("Ajax request failure");
                    this.AutoPersStandardMessageBar.addMessage("error", { text: "The server is not responding", actions: [], closable: true })
                },
            }
            this[dataProviderId].viewModel.getData(requestOptions);
        },

        setData: function (data, id) {
            if (data.length && data[0] && data[0].message) {
                var message = data[0].message;
                this.AutoPersStandardMessageBar.addMessage(message.type, { text: message.text, actions: [], closable: true });
                data.shift();
            }
            if (id === "ComponentListBox") {
                this.toggleAverageTevByPageProgressIndicator(false);
                if (data && !data.length) {
                    this.AverageTevByPageChart.set("dynamicData", []);
                    this.populateComponentListBox(data);
                    return;
                } else {
                    this.loadChart("Component"); //Be sure to trigger when components are unchanged
                }

                this.populateComponentListBox(data);
            } else if (id === "AutoPersStandardMessageBar" && data) {
                this.AutoPersStandardMessageBar.addMessage("warning", { text: data, actions: [], closable: true })
            } else {
                data = this.convertDates(data);
                if (data && data[0] && data[0].values) {
                    var cumul = data[0].values.reduce(function (a, b) {
                        return a + b.y;
                    }, 0);
                    this[id].viewModel.$el.toggleClass("sc-chart-hide-negatives", !cumul);
                }
                this[id].set("dynamicData", data);
            }
        },

        populateComponentListBox: function (data) {
            var flat = [];
            for (var i = 0; i < data.length; i++) {
                var obj = data[i];
                var list = obj.list;
                if (data.length > 1) {
                    flat.push({
                        id: "sc-optGroupUnselectable",
                        label: obj.group
                    });
                }

                for (var j = 0; j < list.length; j++) {
                    var listItem = list[j];
                    var opt = {
                        id: JSON.stringify({ componentid: listItem.id, datasourceid: listItem.DataSourceId }),
                        label: listItem.page + " | " + listItem.componentDatasource
                    }
                    flat.push(opt);
                }
            }
            this.ComponentListBox.set("items", flat);
            if (data.length > 1) {
                this.ComponentListBox.viewModel.$el.addClass('sc-indentable-optgroups');
            } else {
                this.ComponentListBox.viewModel.$el.attr("multiple", "multiple");
            }
            this.ComponentListBox.viewModel.$el.find('option').hover(function (e) {
                var $target = $(e.target);
                $target.attr('title', $target[0].innerText);
            });
        },

        convertDates: function (data) {
            if (!data) {
                return;
            }
            data.forEach(function (series) {
                series.values.map(function (datum) {
                    datum.date && (datum.date = new Date(datum.date + "T12:00:00+0000"));
                    return datum;
                })
            });
            return data;
        },
    });

    return AutoPersonalizationStandard;
});